import 'package:easy_localization/easy_localization.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';

import '../../../config/app_colors.dart';
import '../../../utils/color_extensions.dart';

class CustomBarChart extends StatelessWidget {
  CustomBarChart({super.key});

  final double chartHeight = 120.0;
  final double verticalTicksInterval = 10;

  @override
  Widget build(BuildContext context) {
    return Container(
      child: SizedBox(
        height: this.chartHeight,
      ),
    );
  }

  BarChart getBarChart({barWidth, backgroundColor}) {
    return BarChart(
      BarChartData(
        barGroups: getDataCounts(barWidth),
        backgroundColor: backgroundColor,
        borderData: getBorderData(),
        gridData: getGridData(),
        titlesData: getTitlesData(),
        barTouchData: getBarTouchData(),
        maxY: getNextRoundNumber(getMaxCountsValue(), this.verticalTicksInterval),
      ),
    );
  }

  double getBarWidth(double containerWidth, int barsCount) {
    return 0.7 * (containerWidth / barsCount);
  }

  List<BarChartGroupData> getDataCounts(double barWidth) {
    return [];
  }

  double getMaxCountsValue() {
    return 0.0;
  }

  double getNextRoundNumber(double number, double scale) {
    return scale * ((number ~/ scale).toInt() + 1);
  }

  BarChartGroupData getBarItem(
      {required int x,
      required double value,
      required Color barColor,
      required double barWidth}) {
    final gradient = this.getGradient(barColor);
    final borderColor = barColor.darken(20);

    return BarChartGroupData(
      x: x,
      barRods: [
        BarChartRodData(
          toY: value,
          color: barColor,
          gradient: gradient,
          width: barWidth,
          borderRadius: BorderRadius.all(Radius.zero),
          borderSide: BorderSide(
            color: borderColor,
          ),
        ),
      ],
    );
  }

  LinearGradient getGradient(Color baseColor) {
    return LinearGradient(
      begin: Alignment.topLeft,
      end: Alignment(0.8, 1),
      colors: <Color>[
        baseColor.lighten(30),
        baseColor,
        baseColor.darken(30),
      ],
      tileMode: TileMode.mirror,
    );
  }

  FlBorderData getBorderData() {
    return FlBorderData(
      show: true,
      border: Border.all(
        color: AppColors.borderColor,
        width: 2,
      ),
    );
  }

  FlGridData getGridData() {
    return const FlGridData(
      show: true,
      drawHorizontalLine: true,
      drawVerticalLine: false,
    );
  }

  FlTitlesData getTitlesData() {
    const AxisTitles none = const AxisTitles(
      sideTitles: SideTitles(showTitles: false),
    );

    final AxisTitles verticalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 30,
        getTitlesWidget: getVerticalTitlesWidget,
        interval: this.verticalTicksInterval,
      ),
    );
    final AxisTitles horizontalTitles = AxisTitles(
      sideTitles: SideTitles(
        showTitles: true,
        reservedSize: 20,
        getTitlesWidget: getHorizontalTitlesWidget,
      ),
    );

    return FlTitlesData(
      show: true,
      bottomTitles: horizontalTitles,
      leftTitles: verticalTitles,
      topTitles: none,
      rightTitles: verticalTitles,
    );
  }

  Widget getVerticalTitlesWidget(double value, TitleMeta meta) {
    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 4,
      child: Text(
        value.toInt().toString(),
        style: const TextStyle(
          color: AppColors.mainTextColor1,
          fontSize: 12,
        ),
      ),
    );
  }

  Widget getHorizontalTitlesWidget(double value, TitleMeta meta) {
    final DateFormat formatter = DateFormat('dd/MM');

    final DateTime date = DateTime.fromMillisecondsSinceEpoch(value.toInt());
    final String text = formatter.format(date);

    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: 4,
      child: RotationTransition(
        turns: new AlwaysStoppedAnimation(-30 / 360),
        child: Text(
          text,
          style: const TextStyle(
            color: AppColors.mainTextColor1,
            fontSize: 11,
          ),
        ),
      ),
    );
  }

  BarTouchData getBarTouchData() {
    return BarTouchData(
      enabled: true,
      touchTooltipData: BarTouchTooltipData(
        tooltipBgColor: Colors.transparent,
        tooltipPadding: EdgeInsets.zero,
        tooltipMargin: 2,
        getTooltipItem: (
          BarChartGroupData group,
          int groupIndex,
          BarChartRodData rod,
          int rodIndex,
        ) {
          return BarTooltipItem(
            rod.toY.round().toString(),
            const TextStyle(
              color: AppColors.mainTextColor2,
              fontWeight: FontWeight.bold,
              fontSize: 10,
            ),
          );
        },
      ),
    );
  }
}
